unit UODBC0100;

interface

uses
	Windows, SysUtils, Classes, ComObj, Variants, ADODB;

type
	(* {NX *)
	TODBCbase = class
	private
		OleVar		: OleVariant;
		Msgs		: TStrings;
		LastError	: string;
	protected
		Active		: Boolean;
		procedure	SetLastError(AException: Exception);
    public
		MaxMsgCount: integer;
		constructor	Create(AMsgs: TStrings = nil);
		destructor	Destroy(); override;
		procedure	AddMsg(AMsg: string);
		function	GetLastError(): string;
    end;

	(* f[^x[XNX *)
	TODBCdatabase = class(TODBCbase)
	private
		ConnectString: string;
	protected
		function	GetDatabase(): OleVariant;
    public
		constructor	Create(AMsgs: TStrings = nil);
		destructor	Destroy(); override;
		function	Open(AConStr: string): Boolean;
        procedure	Close();
    end;

type
	(* R[hZbgNX *)
	TODBCRecordSet = class(TODBCbase)
	private
		DB			: TODBCdatabase;
    public
		constructor	Create(ADatabase: TODBCdatabase);
		destructor	Destroy(); override;
		procedure	Execute(AQuery: string);
        procedure	Close();
		function	Open(AQuery: string; ACursorType: CursorTypeEnum = adOpenKeyset; ALockType: LockTypeEnum = adLockOptimistic): Boolean;
        procedure	MoveNext;
        procedure	MovePrev;
        procedure	MoveFirst;
        procedure	MoveLast;
		procedure	Update;
		function	EOF: Boolean;
		function	BOF: Boolean;
    	function	FieldValue(AFieldName: string): Variant; overload;
    	function	FieldValue(AIndex: integer): Variant;    overload;
	end;

var
	Database: TODBCdatabase = nil;

var
	Msg_DatabaseOpened		: string = 'f[^x[Xɐڑ܂B';
    Msg_NotDatabaseOpened	: string = 'f[^x[XɐڑĂ܂B';
    Msg_DatabaseClosed		: string = 'f[^x[Xؒf܂B';

	Msg_RecordsetOpened		: string = 'R[hZbgJ܂B';
    Msg_NotRecordsetOpened	: string = 'R[hZbg͊JĂ܂B';
    Msg_RecordsetClosed		: string = 'R[hZbg܂B';
	Msg_Executed			: string = 'XVnNG[s܂B';
	Msg_ExecuteFailed  		: string = 'XVnNG[̎sɎs܂B';

implementation

uses
	iUtils;

(*############################################################################*)

(*----------------------------------------------------------------------------*)
procedure	TODBCbase.SetLastError(AException: Exception);
begin
	LastError := AException.Message;
end;
(*----------------------------------------------------------------------------*)
constructor	TODBCbase.Create(AMsgs: TStrings = nil);
begin
	inherited Create();
    Msgs := AMsgs;
	OleVar := Unassigned;
	LastError := '';
	Active := False;
	MaxMsgCount := 100;
end;
(*----------------------------------------------------------------------------*)
destructor	TODBCbase.Destroy();
begin
	OleVar := Unassigned;
	inherited;
end;
(*----------------------------------------------------------------------------*)
procedure	TODBCbase.AddMsg(AMsg: string);
begin
	if (Msgs <> nil) and (AMsg <> '') then
    begin
		if Msgs.Count > MaxMsgCount then Msgs.Delete(0);
        Msgs.Add(NowToStr + ' ' + AMsg);
    end;
end;
(*----------------------------------------------------------------------------*)
function	TODBCbase.GetLastError(): string;
begin
	Result := LastError;
end;

(*############################################################################*)

(*----------------------------------------------------------------------------*)
constructor	TODBCdatabase.Create(AMsgs: TStrings = nil);
begin
	inherited Create(AMsgs);
end;

(*----------------------------------------------------------------------------*)
destructor	TODBCdatabase.Destroy();
begin
	Close();
end;

(*----------------------------------------------------------------------------*)
function	TODBCdatabase.GetDatabase(): OleVariant;
begin
	Result := OleVar;
end;

(*----------------------------------------------------------------------------*)
function	TODBCdatabase.Open(AConStr: string): Boolean;
const
	MAGIC_STRING = 'ADODB.Connection';
begin
	ConnectString := AConStr;
	try
    	OleVar := CreateOleObject(MAGIC_STRING);
        OleVar.Open(ConnectString);
		AddMsg(Msg_DatabaseOpened);
        Active := True;
    except
		on E:Exception do
        begin
			OleVar := Unassigned;
            SetLastError(E);
            AddMsg(GetLastError());
			Active := False;
        end;
    end;
	Result := Active;
end;

(*----------------------------------------------------------------------------*)
procedure	TODBCdatabase.Close();
begin
	if Active then
    begin
        OleVar.Close;
    	OleVar := Unassigned;
		Active := False;
		AddMsg(Msg_DatabaseClosed);
    end else
    begin
		AddMsg(Msg_NotDatabaseOpened);
    end;
end;

(*############################################################################*)

(*----------------------------------------------------------------------------*)
{
}
(*----------------------------------------------------------------------------*)
constructor	TODBCRecordSet.Create(ADatabase: TODBCdatabase);
const
	MAGIC_STRING = 'ADODB.Recordset';
begin
    inherited Create(ADatabase.Msgs);
    DB := ADatabase;
    OleVar := CreateOleObject(MAGIC_STRING);
    Active := False;
end;
(*----------------------------------------------------------------------------*)
destructor	TODBCRecordSet.Destroy();
begin
	Close;
    OleVar := Unassigned;
end;
(*----------------------------------------------------------------------------*)
procedure	TODBCRecordSet.Execute(AQuery: string);
begin
	try
		OleVar := DB.GetDatabase.Execute(AQuery);
		AddMsg(Msg_Executed);
    except
		on E:Exception do
        begin
			OleVar := Unassigned;
            SetLastError(E);
            AddMsg(GetLastError());
        end;
    end;
end;
(*----------------------------------------------------------------------------*)
procedure	TODBCRecordSet.Close();
begin
	if Active then
    begin
        OleVar.Close;
    	OleVar := Unassigned;
		Active := False;
		AddMsg(Msg_RecordsetClosed);
    end else
    begin
		AddMsg(Msg_NotRecordsetOpened);
    end;
end;
(*----------------------------------------------------------------------------*)
function	TODBCRecordSet.Open(AQuery: string; ACursorType: CursorTypeEnum = adOpenKeyset; ALockType: LockTypeEnum = adLockOptimistic): Boolean;
begin
	try
		OleVar := DB.GetDatabase.Execute(AQuery);
		AddMsg(Msg_RecordsetOpened);
		Active := True;
    except
		on E:Exception do
        begin
			OleVar := Unassigned;
            SetLastError(E);
            AddMsg(GetLastError());
			Active := False;
        end;
    end;
    Result := Active;
end;

(*----------------------------------------------------------------------------*)
procedure	TODBCRecordSet.MoveNext;
begin
	try
		OleVar.MoveNext;
    except
		on E:Exception do
        begin
            SetLastError(E);
            AddMsg(GetLastError());
        end;
    end;
end;
(*----------------------------------------------------------------------------*)
procedure	TODBCRecordSet.MovePrev;
begin
	try
		OleVar.MovePrevious;
    except
		on E:Exception do
        begin
            SetLastError(E);
            AddMsg(GetLastError());
        end;
    end;
end;
(*----------------------------------------------------------------------------*)
procedure	TODBCRecordSet.MoveFirst;
begin
	try
		OleVar.MoveFirst;
    except
		on E:Exception do
        begin
            SetLastError(E);
            AddMsg(GetLastError());
        end;
    end;
end;
(*----------------------------------------------------------------------------*)
procedure	TODBCRecordSet.MoveLast;
begin
	try
		OleVar.MoveLast;
    except
		on E:Exception do
        begin
            SetLastError(E);
            AddMsg(GetLastError());
        end;
    end;
end;
(*----------------------------------------------------------------------------*)
procedure	TODBCRecordSet.Update;
begin
	try
		OleVar.Update(EmptyParam, EmptyParam);
    except
		on E:Exception do
        begin
            SetLastError(E);
            AddMsg(GetLastError());
        end;
    end;
end;
(*----------------------------------------------------------------------------*)
function	TODBCRecordSet.EOF: Boolean;
begin
	Result := True;
	try
		Result := OleVar.EOF;
    except
		on E:Exception do
        begin
            SetLastError(E);
            AddMsg(GetLastError());
        end;
    end;
end;
(*----------------------------------------------------------------------------*)
function	TODBCRecordSet.BOF: Boolean;
begin
	Result := True;
	try
		Result := OleVar.BOF;
    except
		on E:Exception do
        begin
            SetLastError(E);
            AddMsg(GetLastError());
        end;
    end;
end;
(*----------------------------------------------------------------------------*)
function	TODBCRecordSet.FieldValue(AFieldName: string): Variant;
begin
	Result := Unassigned;
	try
		Result := OleVar.Fields[AFieldName].Value;
    except
		on E:Exception do
        begin
            SetLastError(E);
            AddMsg(GetLastError());
        end;
    end;
end;
(*----------------------------------------------------------------------------*)
function	TODBCRecordSet.FieldValue(AIndex: integer): Variant;
begin
	Result := Unassigned;
	try
		Result := OleVar.Fields[AIndex].Value;
    except
		on E:Exception do
        begin
            SetLastError(E);
            AddMsg(GetLastError());
        end;
    end;
end;
(*----------------------------------------------------------------------------*)
(*----------------------------------------------------------------------------*)
(*----------------------------------------------------------------------------*)
(*----------------------------------------------------------------------------*)
(*----------------------------------------------------------------------------*)
(*----------------------------------------------------------------------------*)
end.
